@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2014 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2016 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -28,8 +28,11 @@ use strict;
 use warnings;
 use Module::Find;
 use NeedRestart::Utils;
+use NeedRestart::CONT;
+use Sort::Naturally;
 
 use constant {
+    NEEDRESTART_PRIO_NOAUTO	=> 0,
     NEEDRESTART_PRIO_LOW	=> 1,
     NEEDRESTART_PRIO_MEDIUM	=> 10,
     NEEDRESTART_PRIO_HIGH	=> 100,
@@ -39,19 +42,25 @@ require Exporter;
 our @ISA = qw(Exporter);
 
 our @EXPORT = qw(
+    NEEDRESTART_PRIO_NOAUTO
     NEEDRESTART_PRIO_LOW
     NEEDRESTART_PRIO_MEDIUM
     NEEDRESTART_PRIO_HIGH
 
     needrestart_ui
+    needrestart_ui_list
     needrestart_interp_check
     needrestart_interp_source
+    needrestart_cont_check
+    needrestart_cont_get
+    needrestart_cont_cmd
 );
 
 our @EXPORT_OK = qw(
     needrestart_ui_register
     needrestart_ui_init
     needrestart_interp_register
+    needrestart_cont_register
 );
 
 our %EXPORT_TAGS = (
@@ -66,9 +75,12 @@ our %EXPORT_TAGS = (
     interp => [qw(
 	needrestart_interp_register
     )],
+    cont => [qw(
+	needrestart_cont_register
+    )],
 );
 
-our $VERSION = '1.1';
+our $VERSION = '2.9';
 my $LOGPREF = '[Core]';
 
 my %UIs;
@@ -81,7 +93,7 @@ sub needrestart_ui_register($$) {
 }
 
 sub needrestart_ui_init($$) {
-    my $debug = shift;
+    my $verbosity = shift;
     my $prefui = shift;
 
     # load prefered UI module
@@ -92,27 +104,38 @@ sub needrestart_ui_init($$) {
     # autoload UI modules
     foreach my $module (findsubmod NeedRestart::UI) {
 	unless(eval "use $module; 1;") {
-	    warn "Error loading $module: $@\n" if($@ && $debug);
+	    warn "Error loading $module: $@\n" if($@ && ($verbosity > 1));
 	}
     }
 }
 
 sub needrestart_ui {
-    my $debug = shift;
+    my $verbosity = shift;
     my $prefui = shift;
 
-    needrestart_ui_init($debug, $prefui) unless(%UIs);
-    my ($ui) = sort { $UIs{$b} <=> $UIs{$a} } keys %UIs;
+    needrestart_ui_init($verbosity, $prefui) unless(%UIs);
+    my ($ui) = sort { ncmp($UIs{$b}, $UIs{$a}) } grep {
+	($UIs{$_} != NEEDRESTART_PRIO_NOAUTO) || ( defined($prefui) && ($prefui eq $_) )
+    } keys %UIs;
 
     return undef unless($ui);
 
-    print STDERR "$LOGPREF Using UI '$ui'...\n" if($debug);
+    print STDERR "$LOGPREF Using UI '$ui'...\n" if($verbosity > 1);
 
-    return $ui->new($debug);
+    return $ui->new($verbosity);
+}
+
+sub needrestart_ui_list {
+    my $verbosity = shift;
+    my $prefui = shift;
+
+    needrestart_ui_init($verbosity, $prefui) unless(%UIs);
+    return (sort { ncmp($UIs{$b}, $UIs{$a}) } keys %UIs);
 }
 
 
 my %Interps;
+my %InterpCache;
 my $idebug;
 
 sub needrestart_interp_register($) {
@@ -132,10 +155,11 @@ sub needrestart_interp_init($) {
     }
 }
 
-sub needrestart_interp_check($$$) {
+sub needrestart_interp_check($$$$) {
     my $debug = shift;
     my $pid = shift;
     my $bin = shift;
+    my $blacklist = shift;
 
     needrestart_interp_init($debug) unless(%Interps);
 
@@ -144,12 +168,18 @@ sub needrestart_interp_check($$$) {
 	    print STDERR "$LOGPREF #$pid is a ".(ref $interp)."\n" if($debug);
 
 	    my $ps = nr_ptable_pid($pid);
-	    my %files = $interp->files($pid);
+	    my %files = $interp->files($pid, \%InterpCache);
 
-	    if(grep {$_ > $ps->start} values %files) {
+	    foreach my $path (keys %files) {
+		next unless(scalar grep { $path =~ /$_/; } @{$blacklist});
+		print  STDERR "$LOGPREF blacklisted: $path\n" if($debug);
+		delete($files{$path});
+	    }
+
+	    if(grep {!defined($_) || $_ > $ps->start} values %files) {
 		if($debug) {
 		    print STDERR "$LOGPREF #$pid uses obsolete script file(s):";
-		    print STDERR join("\n$LOGPREF #$pid  ", '', map {($files{$_} > $ps->start ? $_ : ())} keys %files);
+		    print STDERR join("\n$LOGPREF #$pid  ", '', map {(!defined($files{$_}) || $files{$_} > $ps->start ? $_ : ())} keys %files);
 		    print STDERR "\n";
 		}
 
@@ -183,4 +213,56 @@ sub needrestart_interp_source($$$) {
     return ();
 }
 
+
+my %CONT;
+my $ndebug;
+
+sub needrestart_cont_register($) {
+    my $pkg = shift;
+
+    $CONT{$pkg} = new $pkg($ndebug);
+}
+
+sub needrestart_cont_init($) {
+    $ndebug = shift;
+
+    # autoload CONT modules
+    foreach my $module (findsubmod NeedRestart::CONT) {
+	unless(eval "use $module; 1;") {
+	    warn "Error loading $module: $@\n" if($@ && $ndebug);
+	}
+    }
+}
+
+sub needrestart_cont_check($$$;$) {
+    my $debug = shift;
+    my $pid = shift;
+    my $bin = shift;
+    my $norestart = shift || 0;
+
+    needrestart_cont_init($debug) unless(scalar keys %CONT);
+
+    foreach my $cont (values %CONT) {
+	return 1 if($cont->check($pid, $bin, $norestart));
+    }
+
+    return 0;
+}
+
+sub needrestart_cont_get($) {
+    my $debug = shift;
+
+    return map {
+	my $cont = $_;
+	my $n = ref $cont;
+	$n =~ s/^NeedRestart::CONT:://;
+
+	my %c = $cont->get;
+	
+	map {
+	    ("$n $_" => $c{$_});
+	} sort keys %c;
+    } sort { (ref $a) cmp (ref $b); } values %CONT;
+}
+
 1;
