@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2014 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -83,6 +83,10 @@ sub source {
     my $self = shift;
     my $pid = shift;
     my $ptable = nr_ptable_pid($pid);
+    unless($ptable->{cwd}) {
+	print STDERR "$LOGPREF #$pid: could not get current working directory, skipping\n" if($self->{debug});
+	return ();
+    }
     my $cwd = getcwd();
     chdir($ptable->{cwd});
 
@@ -91,7 +95,17 @@ sub source {
 
     # eat Python's command line options
     my %opts;
-    getopts('BdEhim:ORQ:sStuvVW:x3?c:', \%opts);
+    {
+	local $SIG{__WARN__} = sub { };
+	getopts('BdEhim:ORQ:sStuvVW:x3?c:', \%opts);
+    }
+
+    # skip python -c '...' calls
+    if(exists($opts{c})) {
+	chdir($cwd);
+	print STDERR "$LOGPREF #$pid: uses no source file (-c), skipping\n" if($self->{debug});
+	return undef;
+    }
 
     # extract source file
     unless($#ARGV > -1) {
@@ -114,7 +128,12 @@ sub source {
 sub files {
     my $self = shift;
     my $pid = shift;
+    my $cache = shift;
     my $ptable = nr_ptable_pid($pid);
+    unless($ptable->{cwd}) {
+	print STDERR "$LOGPREF #$pid: could not get current working directory, skipping\n" if($self->{debug});
+	return ();
+    }
     my $cwd = getcwd();
     chdir($ptable->{cwd});
 
@@ -123,7 +142,17 @@ sub files {
 
     # eat Python's command line options
     my %opts;
-    getopts('BdEhim:ORQ:sStuvVW:x3?c:', \%opts);
+    {
+	local $SIG{__WARN__} = sub { };
+	getopts('BdEhim:ORQ:sStuvVW:x3?c:', \%opts);
+    }
+
+    # skip python -c '...' calls
+    if(exists($opts{c})) {
+	chdir($cwd);
+	print STDERR "$LOGPREF #$pid: uses no source file (-c), skipping\n" if($self->{debug});
+	return ();
+    }
 
     # extract source file
     unless($#ARGV > -1) {
@@ -138,6 +167,13 @@ sub files {
 	print STDERR "$LOGPREF #$pid:  reduced ARGV: ".join(' ', @ARGV)."\n" if($self->{debug});
 	return ();
     }
+    print STDERR "$LOGPREF #$pid: source=$src\n" if($self->{debug});
+
+    # use cached data if avail
+    if(exists($cache->{files}->{(__PACKAGE__)}->{$src})) {
+	print STDERR "$LOGPREF #$pid: use cached file list\n" if($self->{debug});
+	return %{ $cache->{files}->{(__PACKAGE__)}->{$src} };
+    }
 
     # prepare include path environment variable
     my %e = nr_parse_env($pid);
@@ -177,6 +213,8 @@ sub files {
     } keys %files;
 
     chdir($cwd);
+
+    $cache->{files}->{(__PACKAGE__)}->{$src} = \%ret;
     return %ret;
 }
 
