@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2014 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -80,6 +80,10 @@ sub source {
     my $self = shift;
     my $pid = shift;
     my $ptable = nr_ptable_pid($pid);
+    unless($ptable->{cwd}) {
+	print STDERR "$LOGPREF #$pid: could not get current working directory, skipping\n" if($self->{debug});
+	return ();
+    }
     my $cwd = getcwd();
     chdir($ptable->{cwd});
 
@@ -88,7 +92,17 @@ sub source {
 
     # eat Ruby's command line options
     my %opts;
-    getopts('SUacdlnpswvy0:C:E:F:I:K:T:W:e:i:r:x:e:d:', \%opts);
+    {
+	local $SIG{__WARN__} = sub { };
+	getopts('SUacdlnpswvy0:C:E:F:I:K:T:W:e:i:r:x:e:d:', \%opts);
+    }
+
+    # skip ruby -e '...' calls
+    if(exists($opts{e})) {
+	chdir($cwd);
+	print STDERR "$LOGPREF #$pid: uses no source file (-e), skipping\n" if($self->{debug});
+	return undef;
+    }
 
     # extract source file
     unless($#ARGV > -1) {
@@ -111,7 +125,12 @@ sub source {
 sub files {
     my $self = shift;
     my $pid = shift;
+    my $cache = shift;
     my $ptable = nr_ptable_pid($pid);
+    unless($ptable->{cwd}) {
+	print STDERR "$LOGPREF #$pid: could not get current working directory, skipping\n" if($self->{debug});
+	return ();
+    }
     my $cwd = getcwd();
     chdir($ptable->{cwd});
 
@@ -120,7 +139,17 @@ sub files {
 
     # eat Ruby's command line options
     my %opts;
-    getopts('SUacdlnpswvy0:C:E:F:I:K:T:W:e:i:r:x:e:d:', \%opts);
+    {
+	local $SIG{__WARN__} = sub { };
+	getopts('SUacdlnpswvy0:C:E:F:I:K:T:W:e:i:r:x:e:d:', \%opts);
+    }
+
+    # skip ruby -e '...' calls
+    if(exists($opts{e})) {
+	chdir($cwd);
+	print STDERR "$LOGPREF #$pid: uses no source file (-e), skipping\n" if($self->{debug});
+	return ();
+    }
 
     # extract source file
     unless($#ARGV > -1) {
@@ -135,6 +164,13 @@ sub files {
 	print STDERR "$LOGPREF #$pid:  reduced ARGV: ".join(' ', @ARGV)."\n" if($self->{debug});
 	return ();
     }
+    print STDERR "$LOGPREF #$pid: source=$src\n" if($self->{debug});
+
+    # use cached data if avail
+    if(exists($cache->{files}->{(__PACKAGE__)}->{$src})) {
+	print STDERR "$LOGPREF #$pid: use cached file list\n" if($self->{debug});
+	return %{ $cache->{files}->{(__PACKAGE__)}->{$src} };
+    }
 
     # prepare include path environment variable
     my %e = nr_parse_env($pid);
@@ -161,6 +197,8 @@ sub files {
     } keys %files;
 
     chdir($cwd);
+
+    $cache->{files}->{(__PACKAGE__)}->{$src} = \%ret;
     return %ret;
 }
 
