@@ -4,7 +4,7 @@
 #   Thomas Liske <thomas@fiasko-nw.net>
 #
 # Copyright Holder:
-#   2013 - 2014 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
+#   2013 - 2015 (C) Thomas Liske [http://fiasko-nw.net/~thomas/]
 #
 # License:
 #   This program is free software; you can redistribute it and/or modify
@@ -26,23 +26,38 @@ package NeedRestart::UI;
 
 use strict;
 use warnings;
+use Text::Wrap qw(wrap $columns);
 use Term::ReadKey;
 
 sub new {
     my $class = shift;
-    my $debug = shift;
+    my $verbosity = shift;
 
     return bless {
-	debug => $debug,
+	verbosity => $verbosity,
 	progress => undef,
     }, $class;
 }
 
+sub wprint {
+    my $self = shift;
+    my $fh = shift;
+    my $sp1 = shift;
+    my $sp2 = shift;
+    my $message = shift;
+
+    # workaround Debian Bug#824564 in Term::ReadKey: pass filehandle twice
+    my ($cols) = GetTerminalSize($fh, $fh);
+    $columns = $cols if($cols);
+
+    print $fh wrap($sp1, $sp2, $message);
+}
+
 sub progress_prep($$$) {
     my $self = shift;
     my ($max, $out) = @_;
 
-    unless($self->{debug}) {
+    unless(($self->{verbosity} != 1) || !(-t *STDERR)) {
 	# restore terminal if required (debconf)
 	unless(-t *STDIN) {
 	    open($self->{fhin}, '<&', \*STDIN) || die "Can't dup stdin: $!\n";
@@ -116,7 +131,7 @@ sub _progress_out {
     $wmsg = length($self->{progress}->{msg}) if(length($self->{progress}->{msg}) < $wmsg);
     my $wbar = $columns - $wmsg - 1;
 
-    printf("%-${wmsg}s [%-${wbar}s]\r", substr($self->{progress}->{msg}, 0, $wmsg), '=' x ($wbar*$self->{progress}->{count}/$self->{progress}->{max}));
+    printf("%-${wmsg}s [%-${wbar}s]\r", substr($self->{progress}->{msg}, 0, $wmsg), '=' x ($wbar*( $self->{progress}->{max} > 0 ? $self->{progress}->{count}/$self->{progress}->{max} : 0 )));
 }
 
 sub _progress_fin {
@@ -141,7 +156,24 @@ sub notice($$) {
 }
 
 
-sub query_pkgs($$$$) {
+sub command() {
+   my $self = shift;
+
+   $self->notice(@_);
+}
+
+
+sub query_pkgs($$$$$) {
+}
+
+sub query_conts($$$$$) {
+}
+
+sub runcmd {
+    my $self = shift;
+    my $cb = shift;
+
+    &$cb;
 }
 
 1;
